import io
import json
import os
import tarfile
from pathlib import Path

import httpx
import typer
import yaml  # type: ignore
from rich.console import Console
from rich.panel import Panel
from rich.syntax import Syntax

from ...contracts.uac import CharmConfig
from ..config import get_token, load_config
from ..git import GitError, get_repo_info

console = Console()

# [Update] 指向 Next.js Store API (本地開發預設值)
# 若上線後，請將此處改為 https://your-store-domain.com/api
DEFAULT_API_BASE = "http://localhost:3000/api"

# Files to exclude from the bundle for security and size.
IGNORE_SET = {
    ".env",
    ".env.local",
    "secrets.json",
    ".git",
    ".DS_Store",
    "__pycache__",
    "venv",
    ".venv",
    "dist",
    "build",
}


def is_ignored(name: str) -> bool:
    if name in IGNORE_SET:
        return True
    if name.endswith((".pyc", ".pyo", ".pyd")):
        return True
    return False


def create_bundle(source_dir: Path) -> bytes:
    file_stream = io.BytesIO()
    with tarfile.open(fileobj=file_stream, mode="w:gz") as tar:
        for root, dirs, files in os.walk(source_dir):
            dirs[:] = [d for d in dirs if not is_ignored(d)]
            for file in files:
                if is_ignored(file):
                    continue
                full_path = Path(root) / file
                arcname = full_path.relative_to(source_dir)
                tar.add(full_path, arcname=str(arcname))
    file_stream.seek(0)
    return file_stream.getvalue()


def push_command(
    path: str = typer.Argument(".", help="Path to the Charm project root"),
    dry_run: bool = typer.Option(False, "--dry-run", help="Preview payload without sending"),
    api_base_override: str = typer.Option(None, "--api-base", help="Override API base URL"),
):
    """
    Register/Publish the Agent to the Charm Registry.
    Uploads source code bundle and links metadata.
    """
    project_path = Path(path).resolve()

    token = get_token()
    if not token:
        console.print("[bold red]Auth Error:[/bold red] Please run [bold]charm auth[/bold] first.")
        raise typer.Exit(code=1)

    yaml_file = project_path / "charm.yaml"
    if not yaml_file.exists():
        console.print(f"[bold red]Error:[/bold red] charm.yaml not found in {project_path}")
        raise typer.Exit(code=1)

    try:
        with open(yaml_file, "r") as f:
            uac_payload = yaml.safe_load(f)
            config = CharmConfig(**uac_payload)
            uac_payload = config.model_dump(mode="json", exclude_none=True)
    except Exception as e:
        console.print(f"[bold red]Config Error:[/bold red] {e}")
        raise typer.Exit(code=1) from e

    try:
        repo_info = get_repo_info(project_path)
    except GitError:
        repo_info = {"url": "", "branch": "main", "commit": "unknown", "is_dirty": "False"}

    if repo_info.get("is_dirty") == "True" and not dry_run:
        console.print(
            "[bold yellow]Warning:[/bold yellow] You have uncommitted changes. Uploading local files anyway."
        )

    metadata_payload = {
        "uac": uac_payload,
        "repo": {
            "url": repo_info["url"],
            "branch": repo_info["branch"],
            "commit": repo_info["commit"],
        },
    }

    with console.status("[bold green]Bundling source code...[/bold green]"):
        bundle_bytes = create_bundle(project_path)
        bundle_size_kb = len(bundle_bytes) / 1024

    if dry_run:
        console.print(f"\n[bold blue]Dry Run:[/bold blue] Bundle Size: {bundle_size_kb:.2f} KB")
        console.print(Syntax(json.dumps(metadata_payload, indent=2), "json", theme="monokai"))
        raise typer.Exit(code=0)

    config_data = load_config()

    api_base = api_base_override or config_data.get("core", {}).get("api_base") or DEFAULT_API_BASE

    api_base = str(api_base).rstrip("/")

    target_url = f"{api_base}/v1/agents"

    console.print(f" Pushing to [underline]{target_url}[/underline]...")

    try:
        with console.status("[bold green]Uploading Bundle & Metadata...[/bold green]"):
            files = {"file": ("source.tar.gz", bundle_bytes, "application/gzip")}
            data = {"metadata": json.dumps(metadata_payload)}

            response = httpx.post(
                target_url,
                files=files,
                data=data,
                headers={"Authorization": f"Bearer {token}"},
                timeout=60.0,
            )

        if response.status_code in [200, 201]:
            resp_data = response.json()
            agent_url = resp_data.get("url", "N/A")

            agent_version = getattr(config.persona, "version", "0.1.0")

            console.print(
                Panel(
                    f"[bold]Agent:[/bold] {config.persona.name}\n"
                    f"[bold]Version:[/bold] {agent_version}\n"
                    f"[bold]Size:[/bold] {bundle_size_kb:.2f} KB\n\n"
                    f"🔗 [link={agent_url}]{agent_url}[/link]",
                    title="[bold green]✔ Successfully Published[/bold green]",
                    border_style="green",
                )
            )
        else:
            console.print(f"[bold red]Server Error ({response.status_code}):[/bold red]")
            try:
                err_msg = response.json().get("error", response.text)
                console.print(f"[red]{err_msg}[/red]")
            except Exception:
                console.print(response.text)
            raise typer.Exit(code=1)

    except Exception as e:
        console.print(f"[bold red]Connection Error:[/bold red] {e}")
        raise typer.Exit(code=1) from e



import typer
import shutil
import os
from pathlib import Path
from rich.console import Console
from importlib.resources import files 

app = typer.Typer(help="Initialize a new Charm agent")
console = Console()

@app.command("init")
def init_command(
    name: str = typer.Argument(..., help="Name of the agent directory"),
    template: str = typer.Option("default", help="Template to use")
):
    """
    Scaffold a new Charm Agent project.
    """
    project_path = Path(name)
    
    if project_path.exists():
        console.print(f"[bold red]Error:[/bold red] Directory '{name}' already exists.")
        raise typer.Exit(1)

    project_path.mkdir(parents=True)
    
    try:
        # Load the default template from the package resources.
        template_source = files("charm.templates").joinpath("charm.default.yaml")
        content = template_source.read_text(encoding="utf-8")
        
        # Write charm.yaml
        target_file = project_path / "charm.yaml"
        target_file.write_text(content, encoding="utf-8")
        
        # Create src/main.py placeholder
        (project_path / "src").mkdir()
        (project_path / "src" / "main.py").write_text("# Your agent code here\n", encoding="utf-8")

        console.print(f"[bold green]✔ Created new agent project: {name}[/bold green]")
        console.print(f"  ├── charm.yaml (Created from template)")
        console.print(f"  └── src/main.py")
        console.print("\nNext step:\n  [cyan]cd[/cyan] " + name + "\n  [cyan]charm validate[/cyan]")

    except Exception as e:
        console.print(f"[bold red]Error loading template:[/bold red] {e}")
        shutil.rmtree(project_path) # Cleanup on failure 
        raise typer.Exit(1)


import typer
import threading
import webbrowser
import json
import socket
from http.server import BaseHTTPRequestHandler, HTTPServer
from rich.console import Console
from ..config import save_auth_data, get_email, get_token, save_token

app = typer.Typer(help="Manage login and authentication")
console = Console()

STORE_URL = "http://localhost:3000"

class OAuthCallbackHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        if self.path == '/callback':
            try:
                content_length = int(self.headers['Content-Length'])
                post_data = self.rfile.read(content_length)
                data = json.loads(post_data.decode('utf-8'))
                
                access_token = data.get("access_token")
                email = data.get("user_email")

                if access_token:
                    save_auth_data(access_token, email)
                    
                    self.send_response(200)
                    self.send_header('Content-type', 'application/json')
                    self.send_header('Access-Control-Allow-Origin', '*') 
                    self.end_headers()
                    self.wfile.write(json.dumps({"received": True}).encode())
                    
                    threading.Thread(target=self.server.shutdown).start()
                else:
                    self.send_error(400, "Missing token")
            except Exception as e:
                self.send_error(500)
    
    def do_OPTIONS(self):
        self.send_response(200)
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'POST, OPTIONS')
        self.send_header('Access-Control-Allow-Headers', 'Content-Type')
        self.end_headers()

    def log_message(self, format, *args):
        return

def find_free_port():
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.bind(('', 0))
        return s.getsockname()[1]

@app.command()
def login():
    """
    Login via browser.
    """
    console.print("[bold blue]Charm CLI Login[/bold blue]")
    
    port = find_free_port()
    server = HTTPServer(('127.0.0.1', port), OAuthCallbackHandler)
    server_thread = threading.Thread(target=server.serve_forever)
    server_thread.start()
    
    login_url = f"{STORE_URL}/cli/login?port={port}"
    
    console.print(f"Opening browser: [underline]{login_url}[/underline]")
    console.print("Waiting for authentication...", style="yellow")
    
    webbrowser.open(login_url)
    server_thread.join()
    
    email = get_email()
    if email:
        console.print(f"[green]Successfully logged in as {email}![/green]")
    else:
        console.print("[red]Login failed.[/red]")

@app.command()
def logout():
    """Clear local credentials."""
    save_auth_data("", "")
    console.print("Logged out.")

@app.command()
def whoami():
    """Show current user."""
    email = get_email()
    token = get_token()
    if token:
        user = email if email else "Unknown User (Token only)"
        console.print(f"Logged in as: [bold cyan]{user}[/bold cyan]")
    else:
        console.print("Not logged in.")

@app.command()
def manual(token: str = typer.Option(..., prompt=True, hide_input=True)):
    """Manually paste a token."""
    save_token(token)
    console.print("Token saved.")



from typing import Any, Dict, List
import inspect
import asyncio 
import json
import base64 
from .base import BaseAdapter
from ..core.logger import logger

# Import MemorySaver for state checkpointing
try:
    from langgraph.checkpoint.memory import MemorySaver
except ImportError:
    MemorySaver = None

try:
    from langchain_core.messages import HumanMessage, AIMessage, SystemMessage, BaseMessage
except ImportError:
    from langchain.schema import HumanMessage, AIMessage, SystemMessage, BaseMessage # type: ignore

class CharmLangGraphAdapter(BaseAdapter):
    """Adapter for LangGraph CompiledGraphs."""

    def _ensure_instantiated(self):
        self._smart_instantiate()

        # Initialize in-memory checkpointer for state management
        self.checkpointer = MemorySaver() if MemorySaver else None

        if not hasattr(self.agent, "invoke"):
            if hasattr(self.agent, "app") and hasattr(self.agent.app, "invoke"):
                print("[Charm] Detected Wrapper Class. Switching to inner '.app' attribute.")
                self.agent = self.agent.app
            elif hasattr(self.agent, "graph") and hasattr(self.agent.graph, "invoke"):
                print("[Charm] Detected Wrapper Class. Switching to inner '.graph' attribute.")
                self.agent = self.agent.graph
        
        # Re-compile the graph with the checkpointer to enable time-travel/state restoration
        if hasattr(self.agent, "checkpointer") and self.agent.checkpointer is None:

             pass

    def _convert_history_to_messages(self, history: List[Dict[str, str]]) -> List[Any]:
        lc_messages = []
        for msg in history:
            role = msg.get("role")
            content = str(msg.get("content", "")).strip()
            
            if not content:
                continue

            if role == "user":
                lc_messages.append(HumanMessage(content=content))
            elif role == "assistant":
                lc_messages.append(AIMessage(content=content))
            elif role == "system":
                lc_messages.append(SystemMessage(content=content))
        return lc_messages

    def invoke(self, inputs: Dict[str, Any], callbacks: List[Any] = None) -> Dict[str, Any]:
        self._pending_inputs = inputs
        
        try:
            self._ensure_instantiated()
        except Exception as e:
             return {
                "status": "error",
                "error_type": "InstantiationError",
                "message": f"Failed to instantiate LangGraph agent: {str(e)}"
            }

        # CompiledStateGraph
        if not hasattr(self.agent, "invoke"):
             return {
                "status": "error",
                "error_type": "ContractViolation",
                "message": (
                    f"Entry point resolved to type '{type(self.agent).__name__}', "
                    "but 'langgraph' adapter expects a CompiledGraph object (missing 'invoke' method).\n"
                    "Did you forget to call `.compile()` on your graph?"
                )
            }

        # [Use a fixed thread ID for the stateless run (state is hydrated manually)
        config = {"configurable": {"thread_id": "charm_session"}}
        if callbacks:
            config["callbacks"] = callbacks

        native_input = inputs.copy()
        
        # State Restoration
        # If the runner/client sent back a saved state snapshot, hydrate the graph.
        raw_state = native_input.pop("__charm_state__", None)
        if raw_state and hasattr(self.agent, "update_state"):
            try:
                logger.debug("[Charm] Hydrating LangGraph state from snapshot...")
                saved_state = json.loads(base64.b64decode(raw_state).decode('utf-8'))
                # We update the state of the graph using the provided values
                self.agent.update_state(config, saved_state)
            except Exception as e:
                logger.warning(f"[Charm] Failed to restore state: {e}")

        history_data = native_input.pop("__charm_history__", None)
        
        if history_data and "messages" in native_input:
             lc_messages = self._convert_history_to_messages(history_data)
             if isinstance(native_input["messages"], list):
                native_input["messages"] = lc_messages + native_input["messages"]
             else:
                native_input["messages"] = lc_messages

        if "input" in native_input and "messages" not in native_input and len(native_input) == 1:
             logger.debug("[Charm] Converting simple 'input' to 'messages' for LangGraph.")
             native_input["messages"] = [HumanMessage(content=str(native_input["input"]))]
             del native_input["input"]


        result = None

        try:
            result = self.agent.invoke(native_input, config=config)
        except Exception as e:
            error_str = str(e).lower()
            if "no synchronous function" in error_str or "async" in error_str:
                logger.info("[Charm] Detected Async Graph. Switching to ainvoke...")
                try:
                    result = self._execute_async_safely(self.agent.ainvoke(native_input, config=config))
                except Exception as async_e:
                    return {"status": "error", "message": f"Async Graph Execution Failed: {str(async_e)}"}
            else:
                return {"status": "error", "message": f"Graph Execution Failed: {str(e)}"}

        # State Snapshot
        # Extract the final state, serialize it, and pass it back to the wrapper.
        charm_state_payload = ""
        if hasattr(self.agent, "get_state"):
            try:
                snapshot = self.agent.get_state(config)
                # We only save the 'values' (keys), not the entire checkpoint metadata
                if snapshot and hasattr(snapshot, "values"):
                    state_json = json.dumps(snapshot.values, default=str)
                    charm_state_payload = base64.b64encode(state_json.encode('utf-8')).decode('utf-8')
            except Exception as e:
                logger.warning(f"[Charm] Failed to snapshot state: {e}")

        try:
            output_str = ""

            if isinstance(result, dict):
                if "messages" in result:
                    messages = result["messages"]
                    if isinstance(messages, list) and len(messages) > 0:
                        last_msg = messages[-1]
                        
                        content = getattr(last_msg, "content", "")
                        additional_kwargs = getattr(last_msg, "additional_kwargs", {})
                        
                        if content and str(content).strip():
                            output_str = str(content)
                        elif hasattr(last_msg, "tool_calls") and last_msg.tool_calls:
                            tools_desc = []
                            for tool in last_msg.tool_calls:
                                t_name = tool.get("name", "tool")
                                t_args = json.dumps(tool.get("args", {}))
                                tools_desc.append(f"🛠️ Call: {t_name}({t_args})")
                            output_str = "\n".join(tools_desc)
                        elif "function_call" in additional_kwargs:
                            fc = additional_kwargs["function_call"]
                            t_name = fc.get("name", "unknown_tool")
                            t_args = fc.get("arguments", "{}")
                            output_str = f"🛠️ Call (Legacy): {t_name}({t_args})"
                        elif hasattr(last_msg, "response_metadata"):
                            meta = last_msg.response_metadata
                            if "prompt_feedback" in meta:
                                output_str = f"Safety Block: {meta['prompt_feedback']}"
                            elif "finish_reason" in meta:
                                output_str = f"Stop Reason: {meta['finish_reason']}"
                            else:
                                output_str = f"(Empty Content. Raw Message: {str(last_msg)})"
                        else:
                            output_str = f"(Unknown Message Format: {str(last_msg)})"
                
                elif "generation" in result:
                    output_str = str(result["generation"])
                elif "result" in result:
                    output_str = str(result["result"])
                else:
                    # Try to find meaningful string values in the dict
                    output_str = json.dumps(result, ensure_ascii=False, default=str)
            
            else:
                output_str = str(result)
            
            if not output_str or not output_str.strip():
                output_str = f"(Agent returned empty content. Result Type: {type(result)})"

            return {
                "status": "success", 
                "output": output_str,
                "charm_state": charm_state_payload # Return state for wrapper to broadcast
            }
            
        except Exception as e:
            return {"status": "error", "message": f"Output Processing Error: {str(e)}"}

    def get_state(self) -> Dict[str, Any]:
        return {}

    def set_tools(self, tools: List[Any]) -> None:
        pass


import os
from pathlib import Path
import tomlkit
from typing import Optional

# Define the global config directory (usually ~/.charm/).
CONFIG_DIR = Path.home() / ".charm"
CONFIG_FILE = CONFIG_DIR / "config.toml"

DEFAULT_CONFIG = {
    "core": {
        "api_base": "https://charm-registry.vercel.app"
    },
    "auth": {
        "token": "",
        "email": ""
    }
}

def _ensure_config_exists():
    """Creates the config file with defaults if it doesn't exist."""
    if not CONFIG_DIR.exists():
        CONFIG_DIR.mkdir(parents=True)
    
    if not CONFIG_FILE.exists():
        doc = tomlkit.document()
        doc.add("core", DEFAULT_CONFIG["core"])
        doc.add("auth", DEFAULT_CONFIG["auth"])
        with open(CONFIG_FILE, "w", encoding="utf-8") as f:
            f.write(tomlkit.dumps(doc))

def load_config() -> tomlkit.TOMLDocument:
    """Parses the TOML config file."""
    _ensure_config_exists()
    try:
        with open(CONFIG_FILE, "r", encoding="utf-8") as f:
            return tomlkit.parse(f.read())
    except Exception:
        return tomlkit.item(DEFAULT_CONFIG)

def save_token(token: str):
    """Retrieve the stored auth token."""
    config = load_config()
    if "auth" not in config:
        config.add("auth", tomlkit.table())
    
    config["auth"]["token"] = token
    
    with open(CONFIG_FILE, "w", encoding="utf-8") as f:
        f.write(tomlkit.dumps(config))

def save_auth_data(token: str, email: str):
    """
    Persists the authentication token and email to disk.
    """
    config = load_config()
    if "auth" not in config:
        config.add("auth", tomlkit.table())
    
    config["auth"]["token"] = token
    config["auth"]["email"] = email
    
    with open(CONFIG_FILE, "w", encoding="utf-8") as f:
        f.write(tomlkit.dumps(config))

def get_token() -> Optional[str]:
    config = load_config()
    return config.get("auth", {}).get("token")

def get_email() -> Optional[str]:
    config = load_config()
    return config.get("auth", {}).get("email")



import ast
import importlib.util
import inspect
import os
import sys
from pathlib import Path

import typer
import yaml  # type: ignore
from rich.console import Console
from rich.panel import Panel

from ...contracts.uac import CharmConfig

console = Console()


def _check_absolute_paths(project_path: Path) -> list:
    """
    Scans python files for hardcoded absolute paths using AST.
    Returns a list of warnings.
    """
    warnings = []
    ignored_dirs = {".venv", "venv", ".git", "__pycache__", "node_modules", "dist", "build"}

    for root, dirs, files in os.walk(project_path):
        dirs[:] = [d for d in dirs if d not in ignored_dirs]

        for file in files:
            if file.endswith(".py"):
                file_path = Path(root) / file
                try:
                    with open(file_path, "r", encoding="utf-8") as f:
                        tree = ast.parse(f.read())

                    for node in ast.walk(tree):
                        if isinstance(node, ast.Constant) and isinstance(node.value, str):
                            val = node.value
                            is_unix_abs = val.startswith("/") and len(val) > 1 and "/" in val[1:]
                            is_win_abs = len(val) > 2 and val[1] == ":" and val[2] in ["\\", "/"]

                            if (is_unix_abs or is_win_abs) and not val.startswith(
                                ("http", "https", "application/", "text/")
                            ):
                                rel_path = file_path.relative_to(project_path)
                                warnings.append(
                                    f"{rel_path}:{node.lineno} -> Suspicious absolute path: '{val}'"
                                )
                except Exception:
                    pass
    return warnings


def _check_entry_point_signature(project_path: Path, entry_point_str: str) -> list:
    """
    Validates that the entry point function accepts the correct arguments.
    """
    errors = []
    if ":" not in entry_point_str:
        return ["Entry point format invalid. Expected 'module:function'"]

    module_name, func_name = entry_point_str.split(":")

    original_sys_path = sys.path[:]
    sys.path.insert(0, str(project_path))

    try:
        module_file = project_path / Path(module_name.replace(".", "/") + ".py")
        if not module_file.exists():
            module_file = project_path / Path(module_name.replace(".", "/") + "/__init__.py")

        if not module_file.exists():
            return [f"Could not find module file for '{module_name}'"]

        spec = importlib.util.spec_from_file_location(module_name, module_file)
        if spec and spec.loader:
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)

            if not hasattr(module, func_name):
                return [f"Function/Object '{func_name}' not found in module '{module_name}'"]

            obj = getattr(module, func_name)

            if callable(obj):
                try:
                    sig = inspect.signature(obj)
                    valid_params = {"inputs", "callbacks"}

                    for name, param in sig.parameters.items():
                        if (
                            name not in valid_params
                            and param.default == inspect.Parameter.empty
                            and param.kind != inspect.Parameter.VAR_KEYWORD
                        ):
                            errors.append(
                                f"Entry point '{func_name}' has an unsupported required argument: '{name}'.\n"
                                f"    Only 'inputs' and 'callbacks' are provided by Charm Runtime."
                            )
                except ValueError:
                    pass

    except Exception as e:
        errors.append(f"Could not statically analyze entry point (Import Error): {e}")
    finally:
        sys.path = original_sys_path

    return errors


def validate_command(path: str = typer.Argument(".", help="Path to the Charm project root")):
    """
    Validate the charm.yaml configuration and check code integrity.
    """
    project_path = Path(path).resolve()
    yaml_file = project_path / "charm.yaml"

    if not yaml_file.exists():
        console.print(f"[bold red] Error:[/bold red] charm.yaml not found in {project_path}")
        console.print("Are you in the right directory?")
        raise typer.Exit(code=2)

    # YAML Schema Validation
    try:
        with open(yaml_file, "r", encoding="utf-8") as f:
            data = yaml.safe_load(f)
        config = CharmConfig(**data)
    except Exception as e:
        console.print(f"[bold red]✖ Configuration Error:[/bold red] {e}")
        raise typer.Exit(code=1) from e

    console.print(
        Panel(
            f"[bold]Agent:[/bold] {config.persona.name} (v{config.persona.version})\n"
            f"[bold]Adapter:[/bold] {config.runtime.adapter.type}",
            title="[bold green]✔ Schema Valid[/bold green]",
            border_style="green",
        )
    )

    # Code Static Analysis (Linting)
    console.print("\n[bold]Running Code Analysis...[/bold]")
    issues_found = False

    # Entry Point Signature
    if config.runtime.adapter.type == "custom":
        ep_errors = _check_entry_point_signature(project_path, config.runtime.adapter.entry_point)
        if ep_errors:
            issues_found = True
            console.print("[bold red]✖ Entry Point Contract Violation:[/bold red]")
            for err in ep_errors:
                console.print(f"  - {err}")
        else:
            console.print("[green]✔ Entry Point Signature looks correct.[/green]")

    # Absolute Paths
    path_warnings = _check_absolute_paths(project_path)
    if path_warnings:
        console.print(
            "\n[bold yellow]⚠ Portability Warnings (Absolute Paths Detected):[/bold yellow]"
        )
        console.print("  [dim]Absolute paths will break when running in the cloud.[/dim]")
        for w in path_warnings[:5]:
            console.print(f"  - {w}")
        if len(path_warnings) > 5:
            console.print(f"  - ... and {len(path_warnings) - 5} more.")
    else:
        console.print("[green]✔ No hardcoded absolute paths detected.[/green]")

    if issues_found:
        console.print("\n[bold red]Validation Failed due to code issues.[/bold red]")
        raise typer.Exit(code=1)

    console.print("\n[bold green]✨ Project is ready to push![/bold green]")



import unittest
from charm.adapters.crewai import CharmCrewAIAdapter

class MockCrew:
    """Simulates a CrewAI Agent for testing purposes."""
    def kickoff(self, inputs):
        if "topic" in inputs:
            return type('Obj', (), {'raw': f"Processed: {inputs['topic']}"})
        return type('Obj', (), {'raw': "Error: Missing topic"})

class TestCrewAIAdapter(unittest.TestCase):
    
    def test_translation(self):
        """Test the input translation logic (input -> topic) for CrewAI."""
        adapter = CharmCrewAIAdapter(MockCrew())
        
        # Action
        result = adapter.invoke({"input": "Hello Crew"})
        
        # Assert
        self.assertEqual(result["status"], "success")
        self.assertIn("Processed: Hello Crew", result["output"])

if __name__ == "__main__":
    unittest.main()


from typing import Any, Dict, List
import inspect
import asyncio
from .base import BaseAdapter
from ..core.logger import logger

class CharmCrewAIAdapter(BaseAdapter):
    """Adapter for CrewAI Framework."""

    def _ensure_instantiated(self):
        self._smart_instantiate()
        if not hasattr(self.agent, "kickoff"):
            if hasattr(self.agent, "crew") and hasattr(self.agent.crew, "kickoff"):
                print("[Charm] Detected Crew Wrapper. Switching to inner '.crew' attribute.")
                self.agent = self.agent.crew

    def _inject_callbacks(self, callbacks: List[Any]):
        if not callbacks: return
        if hasattr(self.agent, "agents"):
            for agent in self.agent.agents:
                if hasattr(agent, "callbacks"):
                    if agent.callbacks is None: agent.callbacks = []
                    agent.callbacks.extend(callbacks)
                if hasattr(agent, "llm"):
                    if hasattr(agent.llm, "callbacks"):
                        if agent.llm.callbacks is None: agent.llm.callbacks = []
                        agent.llm.callbacks.extend(callbacks)

    def _format_history_as_context(self, history: List[Dict[str, str]]) -> str:
        if not history: return ""
        text = "\n\n--- Conversation History ---\n"
        for msg in history:
            role = msg.get("role", "unknown").upper()
            content = str(msg.get("content", "")).strip()
            if not content: continue
            
            text += f"{role}: {content}\n"
        text += "--- End of History ---\n\n"
        return text

    def invoke(self, inputs: Dict[str, Any], callbacks: List[Any] = None) -> Dict[str, Any]:
        self._pending_inputs = inputs
        
        try:
            self._ensure_instantiated()
        except Exception as e:
             return {
                "status": "error",
                "error_type": "InstantiationError",
                "message": f"Failed to instantiate CrewAI agent: {str(e)}"
            }

        if not hasattr(self.agent, "kickoff"):
            return {
                "status": "error",
                "error_type": "ContractViolation",
                "message": (
                    f"Entry point resolved to type '{type(self.agent).__name__}', "
                    "but 'crewai' adapter expects a Crew object (missing 'kickoff' method).\n"
                    "Did you select the wrong adapter type in charm.yaml?"
                )
            }

        if callbacks:
            self._inject_callbacks(callbacks)

        native_input = inputs.copy()
        
        _ = native_input.pop("__charm_state__", None)

        history_data = native_input.pop("__charm_history__", None)
        
        # Advanced Context Injection for CrewAI
        if history_data:
            # Append to input string
            history_str = self._format_history_as_context(history_data)
            if "topic" in native_input and isinstance(native_input["topic"], str):
                native_input["topic"] += history_str
            elif "input" in native_input and isinstance(native_input["input"], str):
                native_input["input"] += history_str

            # Inject structured context into the first Task
            if hasattr(self.agent, "tasks") and self.agent.tasks:
                context_summary = "\n\n### Context from Previous Turns:\n"
                # We limit to last 6 messages to prevent token overflow
                for msg in history_data[-6:]: 
                    role = msg.get("role", "user").upper()
                    content = msg.get("content", "")
                    context_summary += f"- **{role}**: {content}\n"
                
                context_summary += "\nBased on the above context, continue with the current task goal.\n"

                try:
                    first_task = self.agent.tasks[0]
                    # Check if we haven't already injected it (in case of re-runs in same process)
                    if "### Context" not in first_task.description:
                        first_task.description = context_summary + first_task.description
                        logger.debug("[Charm] Injected history context into CrewAI Task #1")
                except Exception as e:
                    logger.warning(f"[Charm] Failed to inject context into task: {e}")

        if isinstance(native_input, str):
             native_input = {"topic": native_input}

        result = None
        try:
            result = self.agent.kickoff(inputs=native_input)
        
        except Exception as e:
            error_msg = str(e).lower()
            if "await" in error_msg or "async" in error_msg or "coroutine" in error_msg:
                logger.info("[Charm] Detected Async Crew requirements. Switching to async execution...")
                if hasattr(self.agent, "akickoff"): 
                    result = self._execute_async_safely(self.agent.akickoff(inputs=native_input))
                elif hasattr(self.agent, "kickoff_async"): 
                    result = self._execute_async_safely(self.agent.kickoff_async(inputs=native_input))
                else:
                    return {"status": "error", "message": f"Async required but no async method found: {e}"}
            else:
                return {"status": "error", "message": str(e)}

        try:
            output_str = ""
            if hasattr(result, "raw"): output_str = result.raw
            else: output_str = str(result)
            
            return {
                "status": "success", 
                "output": output_str,
                "charm_state": "" 
            }
        except Exception as e:
             return {"status": "error", "message": f"Output parsing error: {e}"}